# This is your system's configuration file.
# Use this to configure your system environment (it replaces /etc/nixos/configuration.nix)
{
  inputs,
  outputs,
  lib,
  config,
  pkgs,
  ...
}: {
  # You can import other NixOS modules here
  imports = [
    # If you want to use modules your own flake exports (from modules/nixos):
    # outputs.nixosModules.example

    # Or modules from other flakes (such as nixos-hardware):
    # inputs.hardware.nixosModules.common-cpu-amd
    # inputs.hardware.nixosModules.common-ssd

    # You can also split up your configuration and import pieces of it here:
    # ./users.nix

    # Import your generated (nixos-generate-config) hardware configuration
    ./hardware-configuration.nix

    # Shared config
    ../../shared/nixos/shared_config.nix

    ../../../modules
  ];

  modules = {
    desktop.apps = {
        godot.enable = true;
        cad.enable = true;
        cam.enable = true;
        code.enable = true;
        art.enable = true;
        xiv.enable = true;
        minecraft.enable = true;
        chat.enable = true;
    };
 #   system.boot.grub.enable = true;
    system.audio.enable = true;
  };

  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;

  nixpkgs = {
    # You can add overlays here
    overlays = [
      # Add overlays your own flake exports (from overlays and pkgs dir):
      outputs.overlays.additions
      outputs.overlays.modifications
      outputs.overlays.unstable-packages

      # You can also add overlays exported from other flakes:
      # neovim-nightly-overlay.overlays.default

      # Or define it inline, for example:
      # (final: prev: {
      #   hi = final.hello.overrideAttrs (oldAttrs: {
      #     patches = [ ./change-hello-to-hi.patch ];
      #   });
      # })
    ];
    # Configure your nixpkgs instance
    config = {
      # Disable if you don't want unfree packages
      allowUnfree = true;
    };
  };

  nix = let
    flakeInputs = lib.filterAttrs (_: lib.isType "flake") inputs;
  in {
    settings = {
      # Enable flakes and new 'nix' command
      experimental-features = "nix-command flakes";
      # Opinionated: disable global registry
      flake-registry = "";
      # Workaround for https://github.com/NixOS/nix/issues/9574
      nix-path = config.nix.nixPath;
    };
    # Opinionated: disable channels
    channel.enable = false;

    # Opinionated: make flake registry and nix path match flake inputs
    registry = lib.mapAttrs (_: flake: {inherit flake;}) flakeInputs;
    nixPath = lib.mapAttrsToList (n: _: "${n}=flake:${n}") flakeInputs;
  };

  nixpkgs.config.packageOverrides = pkgs: {
    nur = import (builtins.fetchTarball {
        url = "https://github.com/nix-community/NUR/archive/master.tar.gz";
        sha256 = "1facb9jgayc9dpnvg3696r5a5w8x0791zkp0qdbiadg56w4lax6r";
      }) {
        inherit pkgs;
    };
  };

  services.xserver.enable = true;
  services.xserver = {
    xkb = {
      variant = "";
      layout = "us";
    };
  };

  # Enable the KDE Plasma Desktop Environment.
  services.displayManager.sddm.enable = true;
  services.desktopManager.plasma6.enable = true;

  # TODO: Set your hostname
  networking.hostName = "blackstar";
  networking.firewall.allowedTCPPorts = [ 8000 80 443 5173 ];

  # TODO: Configure your system-wide user settings (groups, etc), add more users as needed.
  users.users = {

    # FIXME: Replace with your username
    aces = {
      # TODO: You can set an initial password for your user.
      # If you do, you can skip setting a root password by passing '--no-root-passwd' to nixos-install.
      # Be sure to change it (using passwd) after rebooting!
      isNormalUser = true;
      # TODO: Be sure to add any other groups you need (such as networkmanager, audio, docker, etc)
      extraGroups = ["wheel" "networkmanager" "docker"];
    };
  };


  # https://nixos.wiki/wiki/FAQ/When_do_I_update_stateVersion
  system.stateVersion = "24.05";

  # Programs installed for all users
  /*environment.systemPackages = with pkgs; [
    # art
    aseprite
    krita

    # code

    # game
    steam
    alvr
    xivlauncher
    modrinth-app

    # cad
    prusa-slicer
    freecad
    (blender.override {
      cudaSupport = true;
    })
    openscad

    # other
    discord
    onlyoffice-bin
    google-chrome

    # sys utils
    gnupg ffmpeg unzip
    mpv
    hyfetch
    dbeaver-bin
    openssl
    pkg-config
    element-desktop
    python311
    python311Packages.pip
    webkitgtk
    libxkbcommon
  ];*/

  #environment.variables = {
   # PKG_CONFIG_PATH="${pkgs.openssl.dev}/lib/pkgconfig";
   # LD_LIBRARY_PATH="${pkgs.wayland}/lib:$LD_LIBRARY_PATH";
  #};

  services.xserver.videoDrivers = ["nvidia"];

  hardware.opengl = {
  enable = true;

      ## radv: an open-source Vulkan driver from freedesktop
      driSupport = true;
      driSupport32Bit = true;
  };

  # Enable CUPS to print documents.
  services.printing.enable = true;


  hardware.nvidia = {
      modesetting.enable = true;

      powerManagement.finegrained = false;

      nvidiaSettings = true;

      package = config.boot.kernelPackages.nvidiaPackages.latest;
  };

  virtualisation.docker.enable = true;
}
